package coral.counters;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import symlib.SymLiteral;
import cern.colt.Arrays;
import coral.util.Interval;

public class MCUtil {
  
  /**
   * Interval format: [{x1_lo,x1_hi},{x2_lo,x2_hi},...]
   * @param domain
   * @return
   */
  
  public static final Pattern DOMAIN_PATTERN = Pattern.compile("\\{(.+?),(.+?)\\}");
  
  public static Interval[] parseDomain(String domain) {
    List<Interval> ivs = new ArrayList<Interval>();
    
    Matcher m = DOMAIN_PATTERN.matcher(domain);
    while(m.find()) {
      double lo = Double.parseDouble(m.group(1));
      double hi = Double.parseDouble(m.group(2));
      ivs.add(new Interval(lo,hi));
    }
    
    return ivs.toArray(new Interval[]{});
  }
  
  public static void main(String[] args) {
    System.out.println(Arrays.toString(parseDomain("[{22.2,33.3},{123.2,33.4},{11.0,44.3}]")));
  }

  public static Interval[] filterIntervals(Interval[] box, Set<SymLiteral> varSet) {
    List<Integer> boxIndexes = new ArrayList<Integer>();
    for (SymLiteral lit : varSet) {
      boxIndexes.add(lit.getId() - 1);
    }
    
    Interval[] intervalsToUse = new Interval[boxIndexes.size()];
    for (int i = 0; i < intervalsToUse.length; i++) {
      intervalsToUse[i] = box[boxIndexes.get(i)];
    }
    return intervalsToUse;
  }

  public static List<Interval[]> filterBoxes(List<Interval[]> boxes,
      Set<SymLiteral> sortedVars) {
    List<Interval[]> filtered = new ArrayList<Interval[]>();
    for (Interval[] box : boxes) {
      filtered.add(filterIntervals(box, sortedVars));
    }
    
    return filtered;
  }

  public static double avg(List<Double> data) {
    double tmp = 0;
    
    for (double d : data) {
      tmp += d;
    }
    
    return tmp / data.size();
  }

  public static double sampleStdev(List<Double> data, double avg) {
    double tmp = 0.0;
    for (double d : data) {
      tmp += Math.pow(d - avg, 2);
    }
    tmp = tmp / (data.size() - 1);
    return Math.sqrt(tmp);
  }
  
  public static double partitionVar(double mean1, double mean2, double var1, double var2) {
    return Math.pow(mean1, 2) * var2 + Math.pow(mean2, 2) * var1 + var1 * var2;
  }
  
  public static class ORComposer {
    public double accumulatedConstants = 0;
    public double meanEstimate = 0;
    public double varianceEstimate = 0;
    
    final int nSamples;
    public ORComposer(int nSamples) {
      this.nSamples = nSamples; 
    }
    
    public void compose(double mean, double var) {
//      System.out.println("composing [" + meanEstimate + "," + varianceEstimate + "] with [" + mean + "," + var + "]");
      if (var == 0) { //store constants 
        accumulatedConstants += mean;
      } else { //compose variance
        varianceEstimate = varianceEstimate + var;// - 2 * (meanEstimate * mean) / (nSamples); 
//      return var1 + var2 + 2*Math.sqrt(var1*var2);
        meanEstimate += mean;
      }
    }
    
    public double[] getResult() {
      return new double[] {accumulatedConstants + meanEstimate, varianceEstimate};
    }
  }
}
