package coral.counters;

import java.util.ArrayList;
import java.util.List;

import coral.PC;
import coral.util.Interval;
import coral.util.Config;
import coral.util.callers.MathematicaCaller;

public class MathematicaMonteCarloCounter implements ModelCounter {
  
  public final MathematicaCaller mc;
  public final boolean normalize; //divide the volume by the length of the domain 
  
  public MathematicaMonteCarloCounter() {
    mc = new MathematicaCaller();
    normalize = false;
  }
  
  public MathematicaMonteCarloCounter(boolean normalize) {
    mc = new MathematicaCaller();
    this.normalize = normalize;
  }

  @Override
  public EstimateResult count(PC pc) {
    throw new RuntimeException("Not implemented yet");
  }
  
  @Override
  public EstimateResult count(PC pc, Interval[] domains) {
    List<Interval[]> box = new ArrayList<Interval[]>();
    box.add(domains);
    return count(pc,box,domains);
  }

  @Override
  public EstimateResult count(PC pc, List<Interval[]> boxes) {
    double estimate = mc.integrateAdaptativeMonteCarlo(pc, boxes);
    if (coral.counters.Main.mcResetRngEachSimulation) {
		mc.reseed(Config.mcSeed);
    }
    return new EstimateResult(estimate, -10000000000000000000.0);
  }

  @Override
  public EstimateResult count(PC pc, List<Interval[]> boxes, Interval[] domains) {
    //we already have the boxes, so we just use the domains to normalize the result
    double tmp = mc.integrateAdaptativeMonteCarlo(pc, boxes);
    if (coral.counters.Main.mcResetRngEachSimulation) {
      mc.reseed(Config.mcSeed);
    }
    if (normalize) {
      for (Interval iv : domains) {
        tmp /= iv.getLength();
      }
    }
    return new EstimateResult(tmp, -10000000000000000000.0);
  }
  
  public void reset() {
    //do nothing, for now
  }

  @Override
  public void reset(String seed) {
    mc.reseed(seed);
  }
}
