package coral.util;

import java.util.List;

public class Interval {

  public final double lo;
  public final double hi;
  
  public Interval(double lo, double hi) {
    this.lo = lo;
    this.hi = hi;
  }
  
  public String toString() {
    return "[" + lo + "," + hi + "]";
  }
  
  public double getLength() {
    return hi - lo;
  }
  
  public static double computeVolume(Interval[] ivs) {
    double total = 1;
    for (Interval iv : ivs) {
      total = total * iv.getLength();
    }
    
    return total;
  }
  
  public static Interval[] mergeIntervals(List<Interval[]> boxes) {
    double[] lows = new double[boxes.get(0).length];
    double[] highs = new double[boxes.get(0).length];
    java.util.Arrays.fill(lows, (double) Double.MAX_VALUE);
    java.util.Arrays.fill(highs, (double) Double.MIN_VALUE);
 
    for (Interval[] ivs : boxes) {
      for (int i = 0; i < ivs.length; i++) {
        Interval iv = ivs[i];
        if (iv != null) {
          lows[i] = Math.min(iv.lo, lows[i]);
          highs[i] = Math.max(iv.hi, highs[i]);
        }
      }
    }
    Interval[] singleBox = new Interval[lows.length];
    for (int i = 0; i < lows.length; i++) {
      singleBox[i] = new Interval(lows[i], highs[i]);
    }
    return singleBox;
  }
}
