package coral.util.callers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.management.ManagementFactory;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import cern.colt.Arrays;
import symlib.SymBool;
import symlib.SymLiteral;
import symlib.parser.ParseException;
import symlib.parser.Parser;
import coral.PC;
import coral.solvers.rand.Util;
import coral.util.Config;
import coral.util.Interval;
import coral.util.visitors.PaverVisitor;

public class RealPaverCaller implements IntervalSolverCaller {

  @Override
  public Map<Integer, Interval> callSolver(PC pc, int timeout, int precision) throws IOException, InterruptedException {                                  
    return callRealPaver(pc, null, timeout, precision);                                                                                         
  }
  
  //TODO refactor this with getPaving
  public static Map<Integer,Interval> callRealPaver(PC pc, Map<Integer,Interval> box, int timeout,int precision) throws IOException, InterruptedException {
    Map<Integer,Interval> varIdToInterval = new TreeMap<Integer,Interval>();
    Object[] processedInfo = processConstraints(pc, box, timeout, precision, 1, false);

    String realPaverInput = (String) processedInfo[1];
    int nVars = (Integer) processedInfo[0];
    Process p = runRealPaver(realPaverInput);
    
    BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
    int currentVar = 0;
    String line = reader.readLine();
    while (line != null) {
      if(line.contains("OUTER BOX") || line.contains("INNER BOX")) { //solution starts now
        currentVar = 1;
      } else if (currentVar > 0 && currentVar <= nVars) {
        Interval iv;
        int varId;
        double hi,lo;
        if(line.contains("=")) {
          String[] groups = line.split("x|=");
//          System.out.println(Arrays.toString(groups));
          hi = Double.valueOf(groups[2]);
          lo = Double.valueOf(groups[2]);
          varId = Integer.valueOf(groups[1].trim());
        } else {
          String[] groups = line.split("\\[|\\]|\\,|x|in");
//          System.out.println(Arrays.toString(groups));
          hi = Double.valueOf(groups[4]);
          lo = Double.valueOf(groups[3]);
          varId = Integer.valueOf(groups[1].trim());
        }
        iv = new Interval(lo, hi);
        varIdToInterval.put(varId, iv);
        currentVar++;
      }
//      System.out.println(line);
      line = reader.readLine();
    }
    p.waitFor();
//    if(varIdToInterval.size() == 0) {
//      System.out.println("empty interval!");
//    }
    
//    System.out.println("pc:" + pc);
    return varIdToInterval;
  }
  
  //TODO we assume that all calls to the same pc will use the same interval. check if this is true 
  public static Map<PC,List<Interval[]>> pavingCache = new HashMap<PC, List<Interval[]>>();
  
  //TODO pass information of the default interval to realpaver
  @Override
  public List<Interval[]> getPaving(PC pc, Interval[] defaultInterval) throws IOException, InterruptedException {
    List<Interval[]> result;
    
    if (Config.cachePavingResults) {
      result = pavingCache.get(pc);
      if (result == null) {
        result = runPaving(pc,defaultInterval);
        pavingCache.put(pc, result);
      } else {
		  //        System.out.println("[realpaver] cache hit!");
      }
    } else {
       result = runPaving(pc,defaultInterval);
    }
    
	  return result;
  }

  private List<Interval[]> runPaving(PC pc, Interval[] defaultInterval)
      throws IOException, InterruptedException {
    //      System.out.println("blabla");
    int timeout = Config.intervalSolverTimeout;
    int precision = Config.intervalSolverPrecision;
    int nBoxes = Config.intervalSolverMaxBoxes;
    List<Interval[]> boxes = new ArrayList<Interval[]>(Config.intervalSolverMaxBoxes);
    
    Object[] processedInfo = processConstraints(pc, null, timeout, precision, nBoxes, true);
    String realPaverInput = (String) processedInfo[1];
    int nVars = (Integer) processedInfo[0];

    // If all variables don't appear in the pc, return a "sparse array". Remember to filter the
    // useful values before using it. 
    int biggestId = 0;
    for (SymLiteral lit : pc.getSortedVars()) { //FIXME there should be a smarter way to do this
      if (biggestId < lit.getId()) {
        biggestId = lit.getId();
      }
    }
    
    if (nVars == 0) { //all clauses contain unsupported operations; return default box
      System.out.println("[realpaver] all clauses contain unsupported operations. returning default box...");
      boxes.add(defaultInterval);
    } else {
      long start = System.nanoTime();
      Process p = runRealPaver(realPaverInput);
  
      BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
      int currentVar = -1;
      String line = reader.readLine();
      Interval[] box = null;
  
      while (line != null) {
  		//      System.out.println("blabla");
        if(line.contains("OUTER BOX") || line.contains("INNER BOX")) { //solution starts now
          currentVar = 0;
  //        box = new Interval[nVars];
          box = new Interval[biggestId];
        } else if (currentVar >= 0 && currentVar < nVars) {
          Interval iv;
          int varId;
          double hi,lo;
          if(line.contains("=")) {
            String[] groups = line.split("x|=");
  //          System.out.println(Arrays.toString(groups));
            hi = Double.valueOf(groups[2]);
            lo = Double.valueOf(groups[2]);
            varId = Integer.valueOf(groups[1].trim());
          } else {
            String[] groups = line.split("\\[|\\]|\\,|x|in");
  		  //          System.out.println(Arrays.toString(groups));
            hi = Double.valueOf(groups[4]);
            lo = Double.valueOf(groups[3]);
            varId = Integer.valueOf(groups[1].trim());
          }
          iv = new Interval(lo, hi);
          box[varId-1] = iv;
          currentVar++;
        } 
        
        if (currentVar == nVars) {
          boxes.add(box);
          currentVar = -1;
          box = null;
        }
        
  //      System.out.println(line);
        line = reader.readLine();
      }
      long end = System.nanoTime();
//      System.out.printf("[realpaver] execution+processing took %f miliseconds\n", (end - start)/1000000.0);
    }
    
    return boxes;
  }


  private static Process runRealPaver(String realPaverInput)
      throws IOException, InterruptedException {
    String currentPID = ManagementFactory.getRuntimeMXBean().getName();
    String filename = "/tmp/realpaverinput" + currentPID; 
    FileWriter fw = new FileWriter(filename);
    BufferedWriter bw = new BufferedWriter(fw);
    bw.write(realPaverInput); 
    bw.close();
    Process p = Runtime.getRuntime().exec(Config.realPaverLocation + " " + filename);
	//	System.out.println(p);
	//	System.out.println(Config.realPaverLocation + " " + filename);
    return p;
  }
  
  public static String generateInput(List<String> clauses, int timeout, int precision, int nboxes, int[] varIds, Interval[] box, boolean paving) {
    
    boolean hasBox = box != null; 
    
    StringBuffer sb = new StringBuffer();
    //OPTIONS
    sb.append(Config.intervalSolverExtraArgs + "\n");
    sb.append("Time = ");
    sb.append(timeout);
    sb.append(";\n");
    sb.append("Branch \n");
    sb.append("precision = 1.0e-");
    sb.append(precision);
    sb.append(",\n number = ");
    sb.append(nboxes);
    sb.append(", \n mode = " + (paving ? "paving":"points") + "; \n Variables \n");
    
    int i = 0;
    for(int id : varIds) {
        sb.append('x');
        sb.append(id);
        sb.append(" in [");
        
        if(hasBox) {
          Interval iv = box[i];
          sb.append(iv.lo);
          sb.append(',');
          sb.append(iv.hi);
        } else {
          sb.append(Config.rangeLO);
          sb.append(',');
          sb.append(Config.rangeHI);
        }
        sb.append("],\n");
        i++;
    }
    
    int pos = sb.length();
    sb.deleteCharAt(pos - 1); //remove last comma
    sb.deleteCharAt(pos - 2);
    sb.append("; \n Constraints \n");
    
    for(String clause : clauses) {
      sb.append(clause);
      sb.append(",\n");
    }
    
    pos = sb.length();
    sb.deleteCharAt(pos - 1); //remove last comma
    sb.deleteCharAt(pos - 2);
    
    sb.append(';');
    
    return sb.toString();
  }
  
  private static Object[] processConstraints(PC pc, Map<Integer,Interval> box,  int timeout, int precision, int nboxes, boolean paving) {
    List<SymBool> coralCons = pc.splitANDs().getConstraints();
    List<SymBool> allowedCons = new ArrayList<SymBool>(coralCons.size());
    List<String> realPaverCons = new ArrayList<String>(coralCons.size());
    PaverVisitor translator = new PaverVisitor();
        
    for(SymBool bool : coralCons) {
      try {
        String clause = bool.accept(translator);
        allowedCons.add(bool);
        realPaverCons.add(clause);
      } catch (UnsupportedOperationException e) {
//        System.out.println("Skipping unsupported operation...");
      }
    }
    
    PC realPaverPC = new PC(allowedCons); //create a new PC without the clauses that contains unsupported operations
    Set<SymLiteral> realPaverVars = realPaverPC.getVars();
    //int[] realPaverVarsIds = Util.extractVarIds(realPaverVars);
    //use all vars
    int[] realPaverVarsIds = Util.extractVarIds(pc.getVars());
    
    String realPaverInput;
    
    if(box != null) {
      Interval[] intervals = new Interval[realPaverVarsIds.length];
      
      for(int i = 0; i < intervals.length; i++) {
        intervals[i] = box.get(realPaverVarsIds[i]);
      }
      
      realPaverInput = generateInput(realPaverCons, timeout, precision, nboxes, realPaverVarsIds, intervals, paving);
    } else { 
      realPaverInput = generateInput(realPaverCons, timeout, precision, nboxes, realPaverVarsIds, null, paving); 
    }
    
    return new Object[]{new Integer(realPaverVarsIds.length),realPaverInput};
  }
  
  public static void main(String args[]) throws ParseException, IOException, InterruptedException {
    PC pc = (new Parser("DEQ(DVAR(ID_1),ADD(DVAR(ID_2),DVAR(ID_4)));DGT(DVAR(ID_4),ADD(DVAR(ID_2),DVAR(ID_3)))")).parsePC();
//    Map<Integer,Interval> map = callRealPaver(pc,null, 3000, 3);
    List<Interval[]> boxes;
    boxes = new RealPaverCaller().getPaving(pc,null);
    for (Interval[] box : boxes) {
      System.out.println(Arrays.toString(box));
    }
    //Benchmark./*pc61*/pc81)
  }
}
