package coral.util.visitors;

import symlib.SymBoolConstant;
import symlib.SymBoolOperations;
import symlib.SymBoolRelational;
import symlib.SymDoubleArith;
import symlib.SymDoubleConstant;
import symlib.SymDoubleLiteral;
import symlib.SymDoubleRelational;
import symlib.SymFloatArith;
import symlib.SymFloatConstant;
import symlib.SymFloatLiteral;
import symlib.SymFloatRelational;
import symlib.SymIntArith;
import symlib.SymIntConstant;
import symlib.SymIntLiteral;
import symlib.SymIntRelational;
import symlib.SymLongConstant;
import symlib.SymLongLiteral;
import symlib.SymLongRelational;
import symlib.SymMathBinary;
import symlib.SymMathUnary;
import coral.util.visitors.interfaces.StringVisitor;

public class MathematicaVistor extends StringVisitor {

  @Override
  protected String visitSymConstant(SymBoolConstant arg) {
    return arg.evalBool() ? "True" : "False";
  }

  @Override
  protected String visitSymRel(SymIntRelational arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymRel(SymLongRelational arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymRel(SymFloatRelational arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymRel(SymDoubleRelational arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymConstant(SymIntConstant arg) {
    return "" + arg.eval();
  }

  @Override
  protected String visitSymIntLiteral(SymIntLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymIntArith(SymIntArith arg) {
    return "(" + arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this) + ")";
  }

  @Override
  protected String visitSymDoubleConst(SymDoubleConstant arg) {
    // Internal`StringToDouble[s]
    String result = toMathematicaScientificNotation(arg.eval());

    return result;
  }

  @Override
  protected String visitSymDoubleLiteral(SymDoubleLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymDoubleArith(SymDoubleArith arg) {
    return "(" + arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this) + ")";
  }

  @Override
  protected String visitSymFloatConst(SymFloatConstant arg) {
    return arg.eval() + "";
  }

  @Override
  protected String visitSymFloatLiteral(SymFloatLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymArith(SymFloatArith arg) {
    return "(" + arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this) + ")";
  }

  @Override
  protected String visitSymLongConst(SymLongConstant arg) {
    return arg.eval() + "";
  }

  @Override
  protected String visitSymLongLiteral(SymLongLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymBoolOperations(SymBoolOperations arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymBoolRelational(SymBoolRelational arg) {
    return arg.getA().accept(this) + arg.getSymbolOp()
        + arg.getB().accept(this);
  }

  @Override
  protected String visitSymMathUnary(SymMathUnary arg) {
    return arg.getMathematicaOp() + "[" + arg.getArg().accept(this) + "]";
  }

  @Override
  protected String visitSymMathBinary(SymMathBinary arg) {
    String args;
    if (arg.getOp() == SymMathBinary.ATAN2) { //Math.atan2(y,x) == ArcTan[x,y]
      args = arg.getArg2().accept(this) + ","
          + arg.getArg1().accept(this); 
    } else {
      args = arg.getArg1().accept(this) + ","
          + arg.getArg2().accept(this);      
    }
    return arg.getMathmematicaOp() + "[" + args + "]";
  }

  public static String toMathematicaScientificNotation(double d) {
    String result;
    String val = "" + d;
    if (val.contains("E")) {
      String[] tmp = val.split("E");
      result = "(" + tmp[0] + " * " + "10^(" + tmp[1] + "))";
    } else {
      result = val;
    }
    return result;
  }

}
