package coral.util.visitors;

import java.util.regex.Pattern;

import symlib.SymBoolConstant;
import symlib.SymBoolOperations;
import symlib.SymBoolRelational;
import symlib.SymDouble;
import symlib.SymDoubleArith;
import symlib.SymDoubleConstant;
import symlib.SymDoubleLiteral;
import symlib.SymDoubleRelational;
import symlib.SymFloatArith;
import symlib.SymFloatConstant;
import symlib.SymFloatLiteral;
import symlib.SymFloatRelational;
import symlib.SymIntArith;
import symlib.SymIntConstant;
import symlib.SymIntLiteral;
import symlib.SymIntRelational;
import symlib.SymLongConstant;
import symlib.SymLongLiteral;
import symlib.SymLongRelational;
import symlib.SymMathBinary;
import symlib.SymMathUnary;
import coral.util.visitors.interfaces.StringVisitor;

public class PaverVisitor extends StringVisitor {

  //regex to match variables and functions
  public static final Pattern PATTERN = Pattern.compile("x|\\w\\(");
  
  @Override
  protected String visitSymConstant(SymBoolConstant arg) {
    throw new UnsupportedOperationException("RealPaver does not support boolean values");
  }

  @Override
  protected String visitSymRel(SymIntRelational arg) {
    if(arg.getOp() == SymIntRelational.NE) {
      throw new UnsupportedOperationException("RealPaver does not support NE");
    }
    
    StringBuilder sb = new StringBuilder();
    
//    sb.append('(');
    sb.append(visitSymInt(arg.getA()));
    
    String op = null;
    
    switch(arg.getOp()) {
      case SymIntRelational.GT:
      case SymIntRelational.GE:
        op = " >= ";
        break;
      case SymIntRelational.EQ:
        op = " = ";
        break;
      case SymIntRelational.LE:
      case SymIntRelational.LT:
        op = " <= ";
        break;
    }
           
    sb.append(op);
    sb.append(visitSymInt(arg.getB()));
//    sb.append(')');
    
    return sb.toString();
  }

  @Override
  protected String visitSymRel(SymLongRelational arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymRel(SymFloatRelational arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymRel(SymDoubleRelational arg) {
    if(arg.getOp() == SymDoubleRelational.NE) {
      throw new UnsupportedOperationException("RealPaver does not support NE");
    }
    
    StringBuilder sb = new StringBuilder();
    
//    sb.append('(');
    sb.append(visitSymDouble(arg.getA()));
    
    String op = null;
    
    switch(arg.getOp()) {
      case SymIntRelational.GT:
      case SymIntRelational.GE:
        op = " >= ";
        break;
      case SymIntRelational.EQ:
        op = " = ";
        break;
      case SymIntRelational.LE:
      case SymIntRelational.LT:
        op = " <= ";
        break;
    }
           
    sb.append(op);
    sb.append(visitSymDouble(arg.getB()));
//    sb.append(')');
    
    return sb.toString();
  }

  @Override
  protected String visitSymConstant(SymIntConstant arg) {
    return arg.toString();
  }

  @Override
  protected String visitSymIntLiteral(SymIntLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymIntArith(SymIntArith arg) {
    
    String op;
    int argOp = arg.getOp();
    if(argOp > SymIntArith.DIV) {
      throw new UnsupportedOperationException("RealPaver does not support " + SymIntArith.symbols[argOp]);
    } else {
      op = SymIntArith.symbols[argOp];
    }
    
    StringBuilder sb = new StringBuilder();
    String a = visitSymInt(arg.getA());
    String b = visitSymInt(arg.getB());
    
    sb.append('(');
    sb.append(a);
    sb.append(" " + op + " ");
    sb.append(b);
    sb.append(')');
    
    return sb.toString();
  }

  @Override
  protected String visitSymDoubleConst(SymDoubleConstant arg) {
    return arg.toString();
  }

  @Override
  protected String visitSymDoubleLiteral(SymDoubleLiteral arg) {
    return "x" + arg.getId();
  }

  @Override
  protected String visitSymFloatConst(SymFloatConstant arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymFloatLiteral(SymFloatLiteral arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymArith(SymFloatArith arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymLongConst(SymLongConstant arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymLongLiteral(SymLongLiteral arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymBoolOperations(SymBoolOperations arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymBoolRelational(SymBoolRelational arg) {
    throw new UnsupportedOperationException("Not implemented (yet)");
  }

  @Override
  protected String visitSymMathUnary(SymMathUnary symDoubleJavaMath) {
    int argOp = symDoubleJavaMath.getOp();
    boolean isLog10 = false;
    String op;
    switch(argOp) {
      case SymMathUnary.ROUND:
        throw new UnsupportedOperationException("RealPaver does not support this operation: " + SymMathUnary.symbols[argOp]);
      case SymMathUnary.LOG10: //RealPaver does not support log10; we will use logn to reproduce it
        isLog10 = true;
        op = SymMathUnary.symbols[SymMathUnary.LOG];
        break;
      default:
        op = SymMathUnary.symbols[argOp];
    }
    
    StringBuilder sb = new StringBuilder();
    String visitedArg = visitSymDouble(symDoubleJavaMath.getArg());
    
    if(isLog10) {
      sb.append('(');
      sb.append(op);
      sb.append('(');
      sb.append(visitedArg);
      sb.append(") / ");
      sb.append(op);
      sb.append("(10)");
      sb.append(')');
    } else {
      sb.append(op);
      sb.append('(');
      sb.append(visitedArg);
      sb.append(')');
    }
    return sb.toString();
  }

  @Override
  protected String visitSymMathBinary(SymMathBinary symDouble) {
    StringBuffer sb = new StringBuffer();

    if(symDouble.getOp() == SymMathBinary.ATAN2) {
      SymDouble arg1 = symDouble.getArg1();
      SymDouble arg2 = symDouble.getArg2();
      
      String y = visitSymDouble(arg1);
      String x = visitSymDouble(arg2);
      
      sb.append("2 * atan((sqrt(("+x + ")^2 + ("+y+")^2) - ("+x+"))/("+y+"))");
      
      
    } else { //RealPaver supports only integers as exponents
      SymDouble arg1 = symDouble.getArg1();
      SymDouble arg2 = symDouble.getArg2();
      String exponent = visitSymDouble(symDouble.getArg2());
      
      if(PATTERN.matcher(exponent).find()) { //there is a variable/function in the exponent - error!;
        throw new UnsupportedOperationException("RealPaver does not support exponentiation with variables or functions present in the exponent");
      } else {
        double numericExponent = arg2.eval();
        boolean expIsNegative = numericExponent < 0;
        sb.append('(');
        
        if(expIsNegative) { //invert the expression
          numericExponent = numericExponent * -1;
          sb.append("1 / ");
        }

        if(numericExponent == 0.5) { //square root 
          sb.append("sqrt(");
          sb.append(visitSymDouble(arg1));
          sb.append(')');
        } else if (numericExponent % 1 == 0) { //exponent is an integer
          sb.append(visitSymDouble(arg1));
          sb.append(" ^ ");
          sb.append((int) numericExponent);
          sb.append(')');
        } else { 
          throw new UnsupportedOperationException("RealPaver does not support real numbers as an exponent: " + numericExponent);
        }
        
        if(expIsNegative) {
          sb.append(')');
        }
        
      }
    }
    return sb.toString();
  }

  @Override
  protected String visitSymDoubleArith(SymDoubleArith arg) {
    String op;
    int argOp = arg.getOp();
    if(argOp > SymDoubleArith.DIV) {
      throw new UnsupportedOperationException("RealPaver does not support " + SymDoubleArith.symbols[argOp]);
    } else {
      op = SymDoubleArith.symbols[argOp];
    }
    
    StringBuilder sb = new StringBuilder();
    String a = visitSymDouble(arg.getA());
    String b = visitSymDouble(arg.getB());
    
    sb.append('(');
    sb.append(a);
    sb.append(" " + op + " ");
    sb.append(b);
    sb.append(')');
    
    return sb.toString();
  }
  
  public static void main(String[] args) throws Exception {
    System.out.println(3.5 % 1);
    
//    PC pc = new Parser(DavidSamples.sample1).parsePC();
//    PaverVisitor pv = new PaverVisitor();
//    for(SymBool bool : pc.getConstraints()) {
//      try{
//      String s = bool.accept(pv);
//      System.out.print(s);
//      System.out.println(",");
//      } catch (Exception e) {
//        System.out.println("Not accepted:" + bool);
//      }
//    }
  }
}

